"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/lib/index.ts
var lib_exports = {};
__export(lib_exports, {
  createUseQueryParams: () => createUseQueryParams
});
module.exports = __toCommonJS(lib_exports);

// src/lib/create-params.svelte.ts
var import_svelte = require("svelte");

// src/lib/adapters/dom.ts
function dom(options = {}) {
  const { windowObj = window, replace = false } = options;
  const replaceState = windowObj.history.replaceState.bind(windowObj.history);
  const pushState = windowObj.history.pushState.bind(windowObj.history);
  const update = replace ? replaceState : pushState;
  return {
    isBrowser: () => typeof window !== "undefined",
    getBrowserUrl: () => windowObj.location,
    updateBrowserUrl: (search, hash) => update(null, "", `${search}${hash}`),
    // Client-side only adapter
    getServerUrl: () => ({ hash: "", search: "" }),
    updateServerUrl: () => {
    }
  };
}

// src/lib/utils.ts
var import_valibot = require("valibot");
function parseObject(schemas, input) {
  const clone = {};
  const keys = /* @__PURE__ */ new Set([...Object.keys(input), ...Object.keys(schemas)]);
  for (const key of keys) {
    const value = input[key];
    clone[key] = key in schemas && schemas[key] ? parseValue(key, schemas[key], value) : (
      /** Value wasn't defined in the schema, pass through as-is */
      value
    );
  }
  return clone;
}
function parseValue(key, schema, value) {
  if (typeof schema === "function")
    return schema(value);
  if (isZodSchema(schema))
    return schema.parse(value);
  if (isValibotSchema(schema))
    return (0, import_valibot.parse)(schema, value);
  throw new Error(
    `Unknown schema type (${typeof schema}) for field: "${key}" (value: ${value})`
  );
}
function isZodSchema(obj) {
  return typeof obj === "object" && obj && "parse" in obj && "safeParse" in obj;
}
function isValibotSchema(obj) {
  return typeof obj === "object" && obj && "async" in obj && "_parse" in obj;
}
function parseQueryParams(params, schemas) {
  const log = console.log;
  console.log = () => {
  };
  console.log({ params });
  console.log = log;
  if (typeof schemas === "function")
    return schemas(params);
  if (isZodSchema(schemas))
    return schemas.parse(params);
  if (isValibotSchema(schemas))
    return (0, import_valibot.parse)(schemas, params);
  return parseObject(schemas, params);
}
function mapValues(object, mapFn) {
  return Object.fromEntries(
    Object.entries(object).map(([key, value]) => [key, mapFn(value)])
  );
}
function debounce(func, delay) {
  if (delay === 0)
    return func;
  let timeoutId;
  return (...args) => {
    clearTimeout(timeoutId);
    timeoutId = setTimeout(() => {
      func(...args);
    }, delay);
  };
}

// src/lib/create-params.svelte.ts
function createUseQueryParams(validators, options = {}) {
  const {
    debounce: delay = 0,
    windowObj = typeof window === "undefined" ? void 0 : window,
    adapter = dom({ windowObj }),
    serialise = (value) => typeof value === "string" ? value : JSON.stringify(value)
  } = options;
  let raw = $state(getQueryParams());
  const query = $derived(parseQueryParams(raw, validators));
  const search = $derived(
    Object.keys(query).length ? `?${new URLSearchParams(query)}` : ""
  );
  function getQueryParams() {
    const getParams = adapter.isBrowser() ? adapter.getBrowserUrl : adapter.getServerUrl;
    const { search: search2 } = getParams();
    const queryParams = new URLSearchParams(search2);
    return Object.fromEntries(queryParams.entries());
  }
  const updateBrowserUrl = debounce(async () => {
    await (0, import_svelte.tick)();
    if (adapter.isBrowser()) {
      adapter.updateBrowserUrl(search, adapter.getBrowserUrl().hash);
    } else {
      adapter.updateServerUrl(search, adapter.getServerUrl().hash);
    }
  }, delay);
  function updateQueryParams() {
    raw = getQueryParams();
  }
  function setQueryParam(key, value) {
    if (value === void 0) {
      const { [key]: _, ...rest } = raw;
      raw = rest;
    } else {
      raw = { ...raw, [key]: serialise(value) };
    }
    updateBrowserUrl();
  }
  let unsubscribe = () => {
  };
  if (windowObj) {
    const replaceState = windowObj.history.replaceState.bind(windowObj.history);
    const pushState = windowObj.history.pushState.bind(windowObj.history);
    windowObj.history.replaceState = (...args) => {
      replaceState(...args);
      updateQueryParams();
    };
    windowObj.history.pushState = (...args) => {
      pushState(...args);
      updateQueryParams();
    };
    windowObj.addEventListener("popstate", updateQueryParams);
    unsubscribe = () => {
      if (windowObj) {
        windowObj.removeEventListener("popstate", updateQueryParams);
        windowObj.history.pushState = pushState;
        windowObj.history.replaceState = replaceState;
      }
    };
  }
  return () => {
    const params = {};
    for (const key of Object.keys(validators)) {
      Object.defineProperty(params, key, {
        enumerable: true,
        get() {
          return query[key];
        },
        set(newValue) {
          setQueryParam(key, newValue);
        }
      });
    }
    return [
      params,
      {
        get raw() {
          return raw;
        },
        get query() {
          return query;
        },
        get search() {
          return search;
        },
        get all() {
          return { ...raw, ...query };
        },
        keys() {
          return Object.keys(validators);
        },
        entries() {
          return Object.entries(validators).map(([key]) => [key, query[key]]);
        },
        set(params2) {
          raw = mapValues(params2, serialise);
          updateBrowserUrl();
        },
        update(params2) {
          raw = { ...raw, ...mapValues(params2, serialise) };
          updateBrowserUrl();
        },
        remove(...params2) {
          raw = Object.fromEntries(
            Object.entries(raw).filter(([key]) => !params2.includes(key))
          );
        },
        unsubscribe() {
          return unsubscribe();
        }
      }
    ];
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createUseQueryParams
});
